import pygame
from math import floor
from textures import textures_list
from random import randint
from random import choice
import rickroll
from audio import *
from pypresence import Presence
from time import time


pygame.init()
resolution = (1280, 720)
win = pygame.display.set_mode(resolution)
pygame.display.set_caption("BlackCraft")
start_time = time()
rpc = Presence(1141070901983592579)
rpc.connect()
buttons = [{"label": "BlackCraft", "url": "https://www.youtube.com/watch?v=LCFRN0NW_g0%22"}]

sound_2 = pygame.mixer.Sound("stilriries_and_pycharm.mp3")
channel_2 = pygame.mixer.Channel(2)
channel_2.play(sound_2, -1)
channel_2.set_volume(0.5)


class Physic:
    def __init__(self, x, y, width, height, acc, max_vel):
        self.x_cord = x
        self.y_cord = y
        self.width = width
        self.height = height
        self.acc = acc
        self.max_vel = max_vel
        self.hor_velocity = 0
        self.ver_velocity = 0
        self.previous_x = 0
        self.previous_y = 0
        self.jumping = False
        self.hitbox = pygame.Rect(self.x_cord, self.y_cord, self.width, self.height)
        self.collision = True
        self.previous_jumping = False

    def physic_tick(self, blocks):
        self.x_cord += self.hor_velocity
        self.y_cord += self.ver_velocity
        self.ver_velocity += 0.7
        self.hitbox = pygame.Rect(self.x_cord, self.y_cord, self.width, self.height)

        for beam in blocks:
            if beam.hitbox.colliderect(self.hitbox):  # cofanie obiektu do miejsca z poprzedniej klatki
                if self.x_cord + self.width >= beam.x_cord + 1 > self.previous_x + self.width:  # kolizja z prawej strony
                    self.x_cord = beam.x_cord - self.width
                    self.hor_velocity = 0
                if self.x_cord <= beam.x_cord + beam.width - 1 < self.previous_x:  # kolizja z lewej strony
                    self.x_cord = beam.x_cord - beam.y_cord
                    self.hor_velocity = 0
                if self.y_cord + self.height >= beam.y_cord + 1 > self.previous_y + self.height:  # kolizja z dołu
                    self.y_cord = beam.y_cord - self.height
                    self.ver_velocity = 0
                    self.jumping = False
                if self.y_cord <= beam.x_cord + beam.width - 1 < self.previous_y:  # kolizja z góry
                    self.y_cord = beam.y_cord + beam.height
                    self.ver_velocity = 0

        self.previous_x = self.x_cord
        self.previous_y = self.y_cord

        if not self.jumping and self.previous_jumping:
            sound_0 = pygame.mixer.Sound(jump_audio)
            channel_0 = pygame.mixer.Channel(0)
            channel_0.play(sound_0)
            channel_0.set_volume(0.6)

        self.previous_jumping = self.jumping
        self.previous_x = self.x_cord
        self.previous_y = self.y_cord


class Player(Physic):
    def __init__(self):
        self.stand_right_img = pygame.image.load("Steve_Mojang_1.png")  # player stand image
        self.stand_left_img = pygame.transform.flip(pygame.image.load("Steve_Mojang_1.png"), True, False)
        self.demage_right_img = pygame.image.load("Steve_Mojang_demage.png")
        self.demage_left_img = pygame.transform.flip(pygame.image.load("Steve_Mojang_demage.png"), True, False)
        self.jump_right_img = pygame.image.load("Steve_Mojang_jump.png")  # player jump image
        self.jump_left_img = pygame.transform.flip(pygame.image.load("Steve_Mojang_jump.png"), True, False)
        self.walk_right_img = [pygame.image.load(f"Steve_Mojang_{x}.png") for x in range(1, 4)]  # player walk images
        self.walk_left_img = [pygame.transform.flip(pygame.image.load(f"Steve_Mojang_{x}.png"), True, False) for x in range(1, 4)]
        self.heart_img = pygame.image.load("heart.png")  # player heart image
        self.heart_black_img = pygame.image.load("heart_black.png")
        width = self.stand_right_img.get_width()  # player width
        height = self.stand_right_img.get_height()  # player height
        super().__init__(0, 300, width, height, 5, 10)
        self.walk_index = 0
        self.direction = 1
        self.last_height = self.y_cord
        self.demage_animation = 0
        self.amount_of_hearths = 5
        self.heart_position = (15, 15)
        self.rickrolled = 0

    def tick(self, keys, blocks):
        self.physic_tick(blocks)

        if self.demage_animation > 0:
            self.demage_animation -= 1
        if keys[pygame.K_a] and self.hor_velocity > self.max_vel * -1:
            self.hor_velocity -= self.acc
        if keys[pygame.K_d] and self.hor_velocity < self.max_vel:
            self.hor_velocity += self.acc
        if keys[pygame.K_SPACE] and self.jumping is False:
            self.ver_velocity -= 13
            self.jumping = True
        if keys[pygame.K_t]:  # teleport
            self.x_cord, self.y_cord = 300, 400
            self.direction = 1
        if self.hor_velocity > 0:
            self.direction = 1
        elif self.hor_velocity < 0:
            self.direction = 0

        if not (keys[pygame.K_a] or keys[pygame.K_d]):
            if self.hor_velocity > 0:
                self.hor_velocity -= self.acc
            elif self.hor_velocity < 0:
                self.hor_velocity += self.acc

        if self.y_cord - self.last_height >= 150 and not self.jumping:
            self.demage_animation = 15
            self.amount_of_hearths -= 1

        if self.x_cord > 600 and self.y_cord + self.height >= 620:
            self.x_cord, self.y_cord = 300, 400

        if not self.jumping:
            self.last_height = self.y_cord

        if self.amount_of_hearths == 0:
            self.amount_of_hearths = 5
            self.x_cord, self.y_cord = 300, 300
            self.direction = 1
            rickroll.rick_roll()
            self.rickrolled += 1

        if self.demage_animation == 15:
            sound = pygame.mixer.Sound(choice(damage_audio))
            channel = pygame.mixer.Channel(1)
            channel.play(sound)
            channel.set_volume(0.9)

    def draw(self, background_width):
        if background_width + resolution[0] / 2 > self.x_cord >= resolution[0] / 2:
            self.x_screen = resolution[0] / 2
        elif self.x_cord >= background_width + resolution[0] / 2:
            self.x_screen = self.x_cord - background_width + resolution[0]
        else:
            self.x_screen = self.x_cord

        if self.demage_animation > 0:
            if self.direction == 0:
                win.blit(self.demage_left_img, (self.x_screen, self.y_cord))
            elif self.direction == 1:
                win.blit(self.demage_right_img, (self.x_screen, self.y_cord))
        elif self.jumping:
            if self.direction == 0:
                win.blit(self.jump_left_img, (self.x_screen, self.y_cord))
            elif self.direction == 1:
                win.blit(self.jump_right_img, (self.x_screen, self.y_cord))
        elif self.hor_velocity != 0:
            if self.direction == 0:
                win.blit(self.walk_left_img[floor(self.walk_index)], (self.x_screen, self.y_cord))
            elif self.direction == 1:
                win.blit(self.walk_right_img[floor(self.walk_index)], (self.x_screen, self.y_cord))
            self.walk_index += 0.2
            if self.walk_index > 3:
                self.walk_index = 0
        else:
            if self.direction == 0:
                win.blit(self.stand_left_img, (self.x_screen, self.y_cord))
            elif self.direction == 1:
                win.blit(self.stand_right_img, (self.x_screen, self.y_cord))

        for i in range(0, 5):
            x_position = i * 60 + self.heart_position[0]
            hear_position = (x_position, self.heart_position[1])
            if self.amount_of_hearths > i:
                win.blit(self.heart_img, hear_position)
            else:
                win.blit(self.heart_black_img, hear_position)

class Block:
    def __init__(self, x, y, texture_index: int):
        self.block_img = pygame.image.load(textures_list[texture_index])
        self.x_cord = x
        self.y_cord = y
        self.width = self.block_img.get_width()
        self.height = self.block_img.get_height()
        self.hitbox = pygame.Rect(self.x_cord, self.y_cord, self.width, self.height)

    def draw(self, background_x):
        win.blit(self.block_img, (self.x_cord + background_x, self.y_cord))


class Moving_Block(Block):
    def __init__(self, x, y, texture_index, start_x, end_x, speed):
        self.start_x = start_x
        self.end_x = end_x
        self.speed = speed
        super().__init__(x, y, texture_index)
        self.dimention = 0
        self.position = self.start_x
        self.hitbox = pygame.Rect(self.position, self.y_cord, self.width, self.height)

    def tick(self, player):
        self.hitbox = pygame.Rect(self.position, self.y_cord, self.width, self.height)

        if self.dimention == 0:
            self.position += self.speed
        else:
            self.position -= self.speed
        if self.position <= self.start_x:
            self.dimention = 0
        elif self.position >= self.end_x:
            self.dimention = 1

        if player.hitbox.colliderect(self.hitbox):
            if self.dimention == 0:
                player.x_cord += self.speed
            else:
                player.x_cord -= self.speed

    def draw(self, background_x):
        win.blit(self.block_img, (self.position + background_x, self.y_cord))


class Background:
    def __init__(self):
        self.x_cord = 0
        self.y_cord = 0
        self.image = pygame.image.load("skyv22.png")
        self.width = self.image.get_width()

    def tick(self, player):
        if self.width - resolution[0] / 2 > player.x_cord >= resolution[0] / 2:
            self.x_cord -= player.hor_velocity
        elif player.x_cord >= self.width - resolution[0] / 2:
            self.x_cord = - self.width + resolution[0]
        else:
            self.x_cord = 0

    def draw(self):
        win.blit(self.image, (self.x_cord, self.y_cord))


class Sun:
    def __init__(self):
        self.sun = pygame.image.load("sun.png")
        self.x_cord = 990
        self.y_cord = 40
        self.width = self.sun.get_width()
        self.height = self.sun.get_height()

    def draw(self):
        win.blit(self.sun, (self.x_cord, self.y_cord))

def main():
    run = True  # app main loop
    blocks = [
        Block(500, 520, 0),
        Block(1300, 420, 2),
        Block(1900, 320, 3),
        Block(2200, 320, 4),
        Block(2600, 220, 0),
        Block(3000, 320, 1),
        Block(3300, 420, 2),
        Block(3700, 320, 3),
        Moving_Block(4200, 420, 4, 4200, 4300, 6),
        Moving_Block(4300, 420, 4, 4300, 4400, 6),
        Block(4800, 320, 0),
        Block(5100, 220, 1),
        Block(5600, 320, 4),
        Block(6000, 220, 0),
        Moving_Block(6300, 320, 1, 6300, 6500, 6),
        Block(6800, 420, 3),
        Moving_Block(7200, 420, 4, 7200, 7300, 8),
        Block(7600, 320, 0),
        Block(8000, 420, 1),
        Block(8300, 320, 2),
        Block(8700, 220, 3),
        Block(9000, 220, 4),
        Moving_Block(9400, 320, 3, 9400, 9500, 4),
        Moving_Block(9500, 320, 3, 9500, 9600, 4),
        Block(10100, 320, 3),
        Block(10200, 320, 3),
        Block(10300, 320, 3),
        Moving_Block(800, 420, 4, 800, 1000, 4),
        Moving_Block(1600, 420, 2, 1600, 1800, 10),
    ]
    for i in range(0, 500):
        blocks.append(Block(i * 100, 620, randint(0, len(textures_list) - 1)))
    player = Player()
    background = Background()
    sun = Sun()
    while run:
        fps = 1000 / pygame.time.Clock().tick(120)  # app fps
        fps = int(round(fps, 0))
        if fps == 111 or fps == 125 or fps == 100:
            fps = 120
        amount_of_fps = pygame.font.Font.render(pygame.font.SysFont("arial", 50), f"FPS: {fps}", True, (255, 255, 255))
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run = False
        keys = pygame.key.get_pressed()

        player.tick(keys, blocks)
        background.tick(player)

        background.draw()
        sun.draw()
        win.blit(amount_of_fps, (15, 70))
        player.draw(background.width)
        for block in blocks:
            block.draw(background.x_cord)
        for block in blocks:
            try:
                block.tick(player)
            except Exception:
                pass

        rpc.update(state="BETAv5", details=f"Hearths: {player.amount_of_hearths}, rickrolled {player.rickrolled} times", large_image="foo_new",
                   large_text="Lessgooo!!", start=int(start_time), buttons=buttons)
        pygame.display.update()


if __name__ == "__main__":
    main()
